<?php

namespace Drupal\generative_summary\Controller;

use Drupal\Core\Controller\ControllerBase;
use OpenAI\Client;
use Rajentrivedi\TokenizerX\TokenizerX;

class GenerativeSummaryController extends ControllerBase {

  protected function countTokens($text): int {
    return TokenizerX::count($text);
  }

  /**
   * Split text into smaller chunks.
   *
   * @param array|string $q
   * @param mixed $maxChunkTokens
   * @return array $segments
   */
  public function textSplitter(array|string $q, mixed $maxChunkTokens): array {
    // Treat $q as plain text.
    $sentences = \explode(".", $q);
    $currentSegment = "";
    $currentTokens = 0;

    foreach ($sentences as $sentence) {
      $tokens = $this->countTokens($sentence);
      if ($currentTokens + $tokens > $maxChunkTokens) {
        $segments[] = \trim($currentSegment);
        $currentSegment = "";
        $currentTokens = 0;
      }
      // Prepend "." when adding a new sentence from 2nd round.
      if (!empty($currentSegment)) {
        $currentSegment .= ". ";
      }
      $currentSegment .= $sentence;
      $currentTokens += $tokens;
    }

    // For the remaining part of the text.
    if (!empty($currentSegment)) {
      $segments[] = \trim($currentSegment);
    }
    return $segments;
  }

  protected function isHtml($string): bool {
    return \preg_match("/<[^<]+>/", $string, $m) != 0;
  }


  /**
   * Set credentials.
   *
   * @param array $options
   *   The client build options.
   * @return \OpenAI\Client
   *   The OpenAI client.
   */
  public function openAiClient($options): Client {
    try {
      if ($options['openai_api_service'] == 'azure_openai') {
        $client = \OpenAI::factory()
          ->withBaseUri($options['openai_api_base_url'])
          ->withHttpHeader('api-key', $options['openai_api_key'])
          ->withQueryParam('api-version', $options['openai_api_version'])
          ->make();
      }
      else {
        $org = !empty($options['openai_api_org']) ? $options['openai_api_org'] : NULL;
        if (!empty($options['openai_api_key'])) {
          $client = \OpenAI::client($options['openai_api_key'], $org);
        }
        else {
          \Drupal::messenger()->addMessage('Missing or incorrect OpenAI API Key', 'error');
        }
      }
    }
    catch (\Exception $ex) {
      \Drupal::messenger()->addMessage($ex->getMessage(), 'error');
    }
    return $client;
  }

  /**
   * @param $options
   * @param $body
   * @return string
   */
  public function doRequest($options, $body): string
  {
    $client = $this->openAiClient($options);
    $payload = [
      'model' => $options['openai_model'],
      'messages' => [
        [
          'role' => 'system',
          'content' => $options['custom_prompt_system_role'],
        ],
        [
          'role' => 'user',
          'content' => $options['custom_prompt_user_role'] . $body,
        ],
      ],
      'temperature' => 0.5,
      'max_tokens' => \intval($options['max_length']),
    ];

    try {
      // Use the "openai-php/client" library to make the API call.
      $response = $client->chat()->create($payload);

      // Extract the summarized content from the response.
      $summary = $response->choices[0]->message->content ?? '';

      return $summary;
    } catch (\Exception $e) {
      \Drupal::logger('generative_summary')->error('Error generating summary: ' . $e->getMessage());
      return '';
    }
  }

  private function generateSummary($body, $options): string {
    $chunks = $this->textSplitter(\strip_tags($body), $options['max_chunk_tokens']);

    return $this->doRequest($options, $chunks[0]);
  }

  public function generateSummaryFromText($body, $max_length, $max_sentences, $custom_prompt_system_role, $custom_prompt_user_role): string {
    $options = [];
    // Get global settings as fallback.
    $config = \Drupal::config('generative_summary.settings');
    $options['openai_api_service'] = $config->get('openai_api_service');
    $options['openai_api_key'] = $config->get('openai_api_key');
    $options['openai_api_base_url'] = $config->get('openai_api_base_url');
    $options['openai_api_version'] = $config->get('openai_api_version');
    $options['openai_api_org'] = $config->get('openai_api_org');
    $options['openai_model'] = $config->get('openai_model');

    $options['min_length'] = !empty($min_length) ? $min_length : $config->get('default_min_length');
    $options['max_length'] = !empty($max_length) ? $max_length : $config->get('default_max_length');
    $options['max_sentences'] = !empty($max_sentences) ? $max_sentences : $config->get('default_max_sentences');
    $options['max_chunk_tokens'] = !empty($max_chunk_tokens) ? $max_chunk_tokens : $config->get('default_max_chunk_tokens');
    $options['custom_prompt_system_role'] = !empty($custom_prompt_system_role) ? $custom_prompt_system_role : $config->get('default_custom_prompt_system_role');
    $options['custom_prompt_user_role'] = !empty($custom_prompt_user_role) ? $custom_prompt_user_role : $config->get('default_custom_prompt_user_role');

    // Replace tokens: {{ length }} and {{ sentences }}.
    $options['custom_prompt_user_role'] = \str_replace('{{ length }}', $options['max_length'], $options['custom_prompt_user_role']);
    $options['custom_prompt_user_role'] = \str_replace('{{ sentences }}', $options['max_sentences'], $options['custom_prompt_user_role']);

    return $this->generateSummary($body, $options);
  }

}
