<?php

namespace Drupal\generative_summary\Plugin\Field\FieldWidget;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\generative_summary\Controller\GenerativeSummaryController;
use Drupal\text\Plugin\Field\FieldWidget\TextareaWithSummaryWidget;

/**
 * Plugin implementation of the 'textarea_with_generative_summary' widget.
 *
 * -@FieldWidget(
 *   id = "textarea_with_generative_summary",
 *   label = @Translation("Text area with a generative summary"),
 *   field_types = {
 *     "text_with_summary"
 *   }
 * )
 */
class TextareaWithGenerativeSummaryWidget extends TextareaWithSummaryWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $field_instance = $items->getFieldDefinition();
    $enabled = $field_instance->getThirdPartySetting('generative_summary', 'generative_summary_enabled');

    if ($enabled) {
      $min_length = $field_instance->getThirdPartySetting('generative_summary', 'min_length');
      $max_length = $field_instance->getThirdPartySetting('generative_summary', 'max_length');
      $max_sentences = $field_instance->getThirdPartySetting('generative_summary', 'max_sentences');
      $max_chunk_tokens = $field_instance->getThirdPartySetting('generative_summary', 'max_chunk_tokens');
      $custom_prompt_system_role = $field_instance->getThirdPartySetting('generative_summary', 'custom_prompt_system_role');
      $custom_prompt_user_role = $field_instance->getThirdPartySetting('generative_summary', 'custom_prompt_user_role');

      $field_name = $field_instance->getName();
      $wrapper_id = 'edit-summary-wrapper-' . $field_name . '-' . $delta;
      $element['summary']['#prefix'] = '<div id="' . $wrapper_id . '" class="js-text-summary-wrapper text-summary-wrapper">';
      $element['summary']['#suffix'] = '</div>';

      $element['generate_summary'] = [
        '#type' => 'button',
        '#name' => 'button-' . $wrapper_id,
        '#value' => $this->t('Generate Summary'),
        '#prefix' => '<div id="button-' . $wrapper_id . '" class="js-text-summary-wrapper text-summary-wrapper">',
        '#suffix' => '<div class="hidden messages"></div></div>',
        '#ajax' => [
          'callback' => [$this, 'ajaxGenerateSummary'],
          'wrapper' => $wrapper_id,
          'effect' => 'fade',
        ],
        '#weight' => -9,
        '#options' => [
          'field_name' => $field_name,
          'delta' => $delta,
          'min_length' => $min_length,
          'max_length' => $max_length,
          'max_sentences' => $max_sentences,
          'max_chunk_tokens' => $max_chunk_tokens,
          'custom_prompt_system_role' => $custom_prompt_system_role,
          'custom_prompt_user_role' => $custom_prompt_user_role,
        ],
      ];
    }
    return $element;
  }

  public static function ajaxGenerateSummary(array $form, FormStateInterface $form_state): AjaxResponse {
    $triggering_element = $form_state->getTriggeringElement();
    $field_name = $triggering_element['#options']['field_name'];
    $delta = $triggering_element['#options']['delta'];

    $field_values = $form_state->getValue($field_name);
    $body = $field_values[0]['value'];

    $min_length = $triggering_element['#options']['min_length'];
    $max_length = $triggering_element['#options']['max_length'];
    $max_sentences = $triggering_element['#options']['max_sentences'];
    $max_chunk_tokens = $triggering_element['#options']['max_chunk_tokens'];
    $custom_prompt_system_role = $triggering_element['#options']['custom_prompt_system_role'];
    $custom_prompt_user_role = $triggering_element['#options']['custom_prompt_user_role'];

    $response = new AjaxResponse();

    if (\strlen($body) < $min_length) {
      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'text',
        [\t('Please write some text before generating a summary. The minimum length is @min_length characters.', ['@min_length' => $min_length])]
      ));
      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'removeClass',
        ['hidden']
      ));
      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'addClass',
        ['messages--error']
      ));

      return $response;
    }
    else {
      $controller = new GenerativeSummaryController();
      $summary = $controller->generateSummaryFromText($body, $max_length, $max_sentences, $custom_prompt_system_role, $custom_prompt_user_role);

      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'addClass',
        ['hidden']
      ));
      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'removeClass',
        ['messages--error']
      ));
      $response->addCommand(new InvokeCommand(
        \sprintf('#button-edit-summary-wrapper-%s-%d .messages', $field_name, $delta),
        'text',
        ['']
      ));
      $response->addCommand(new InvokeCommand(
        \sprintf('textarea[name="%s[%d][summary]"]', $field_name, $delta),
        'val',
        [$summary]
      ));
    }

    return $response;
  }

}
